"""
The tests in this module are based on those from Python
https://github.com/python/cpython/tree/master/Lib/test/test_json

Licensed under the Python Software Foundation License Version 2.
Copyright © 2001-2020 Python Software Foundation. All rights reserved.
Copyright © 2000 BeOpen.com . All rights reserved.
Copyright © 1995-2000 Corporation for National Research Initiatives . All rights reserved.
Copyright © 1991-1995 Stichting Mathematisch Centrum . All rights reserved.

Converted from Unittest to PyTest using `unittest2pytest` 0.5.dev0 by Hartmut Goebel
https://github.com/pytest-dev/unittest2pytest

"""
